/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function() {

    /**
     * Padding zero.
     * @private
     * @param {number} number - If number is lesser than 10, padding 0 front of number
     */
    function zeroPadding(number) {
        var str = "";

        if (number < 10) {
            str += "0";
        }
        str += number;

        return str;
    }

    /**
     * Returns Time String.
     * @private
     * @param {number} time - If time is 100 seconds, this function returns "00h 01m 40s".
     */
    function getTimeString(time) {
        var timeStr = "",
            hour,
            min,
            sec;

        hour = zeroPadding(parseInt(time / 3600));
        time %= 3600;
        timeStr += hour + "h ";

        min = zeroPadding(parseInt(time / 60));
        time %= 60;
        timeStr += min + "m ";

        sec = zeroPadding(time);
        timeStr += sec + "s";

        return timeStr;
    }

    /**
     * Creates and Returns DIV object.
     * @private
     * @param {string} className - class name for creating DIV object
     */
    function createDiv(className) {
        var div = document.createElement("div");

        div.className = className;

        return div;
    }

    /**
     * Adds Timer Item to Timer List.
     * @private
     * @param {string} title - Timer's title.
     * @param {string} time - Timer's time.
     */
    function addTimer(title, time) {
        /**
         * timerWorker - Web Worker object.
         * timerList - The timer list item container
         * listItem - The DIV element about timer list item
         * listItemHeader - The DIV element about item header
         * listItemTitle - The DIV element about item title
         * listItemTime - The DIV element about item time
         * listItemDeleteButton - The DIV element about delete button
         * listItemProgress - The DIV element about progress bar container
         * listItemProgressBar - The DIV element about progress bar
         * listItemButtons - The DIV element about item footer
         * listItemStartButton - The DIV element about start button
         * listItemResetButton - The DIV element about reset button
         */
        var timerWorker = new Worker("./js/timerWorker.js"),
            timerList = document.querySelector("#timer-list"),
            listItem,
            listItemHeader,
            listItemTitle,
            listItemTime,
            listItemDeleteButton,
            listItemProgress,
            listItemProgressBar,
            listItemButtons,
            listItemStartButton,
            listItemResetButton;

        //Creates list item container
        listItem = createDiv("list-item");

        //Creates item header container
        listItemHeader = createDiv("item-header");

        //Creates title, time DIV and delete button
        listItemTitle = createDiv("item-title");
        listItemTitle.appendChild(document.createTextNode(title));
        listItemDeleteButton = createDiv("item-delete-btn");
        listItemTime = createDiv("item-time");
        listItemTime.appendChild(document.createTextNode(getTimeString(time)));

        //Appends item header's children
        listItemHeader.appendChild(listItemTitle);
        listItemHeader.appendChild(listItemDeleteButton);
        listItemHeader.appendChild(listItemTime);

        //Creates progress bar
        listItemProgress = createDiv("item-progress");
        listItemProgressBar = createDiv("progress-bar");
        listItemProgress.appendChild(listItemProgressBar);

        //Creates item footer container
        listItemButtons = createDiv("item-buttons");

        //Creates start, reset Button
        listItemStartButton = createDiv("item-button-start");
        listItemResetButton = createDiv("item-button-reset");

        //Appends item footer's children
        listItemButtons.appendChild(listItemStartButton);
        listItemButtons.appendChild(listItemResetButton);

        //Appends list item's children
        listItem.appendChild(listItemHeader);
        listItem.appendChild(listItemProgress);
        listItem.appendChild(listItemButtons);

        //Binds delete button click event listener
        listItemDeleteButton.addEventListener("click", function() {
            timerWorker.terminate();
            timerList.removeChild(listItem);
        });

        //Binds start button click event listener
        listItemStartButton.addEventListener("click", function() {
            timerWorker.postMessage({
                message: "START"
            });
        });

        //Binds reset button click event listener
        listItemResetButton.addEventListener("click", function() {
            timerWorker.postMessage({
                message: "RESET"
            });
        });

        //Binds timerWorker onMessage event listener
        timerWorker.onmessage = function(e) {
            if (e.data === "START") {
                // When timerWorker send "START" message, change start button image to pause icon.
                listItemStartButton.style.backgroundImage = "url(\"./image/pause.png\")";
            } else if (e.data === "PAUSE") {
                // When timerWorker send "PAUSE" message, change start button image to start icon.
                listItemStartButton.style.backgroundImage = "url(\"./image/start.png\")";
            } else if (e.data === "RESET") {
                // When timerWorker send "RESET" message, change start button image to start icon.
                listItemStartButton.style.backgroundImage = "url(\"./image/start.png\")";
                // Resets time and progress bar.
                listItemTime.innerHTML = getTimeString(time);
                listItemProgressBar.style.width = "0%";
            } else {
                // When timerWorker send time data, change time and progress bar in list item.
                listItemTime.innerHTML = getTimeString(e.data);
                listItemProgressBar.style.width = ((time - e.data) / time) * 100 + "%";
            }
        };

        //Appends list item to timer list
        timerList.appendChild(listItem);

        //Sends message to timerWorker
        timerWorker.postMessage({
            message: "INIT",
            time: time
        });
    }



    /**
     * Clears contents of Timer Add Page.
     * @private
     */
    function clearAddPage() {
        document.querySelector("#add-timer-title").value = "";
        document.querySelector("#add-timer-hour").value = "00";
        document.querySelector("#add-timer-min").value = "00";
        document.querySelector("#add-timer-sec").value = "00";
    }



    /**
     * Sets Select Options in Timer Add Page.
     * @private
     */
    function setOptionList() {
        var i,
            option1,
            option2,
            option3,
            optionText;

        for (i = 0; i < 60; i++) {
            optionText = zeroPadding(i);

            if (i < 24) {
                option1 = document.createElement("option");
                option1.text = optionText;
                document.querySelector("#add-timer-hour").options.add(option1);
            }
            option2 = document.createElement("option");
            option3 = document.createElement("option");
            option2.text = optionText;
            option3.text = optionText;
            document.querySelector("#add-timer-min").options.add(option2);
            document.querySelector("#add-timer-sec").options.add(option3);
        }
    }

    /**
     * Sets Sample Timers.
     * @private
     */
    function setSampleTimers() {
        addTimer("Hello Tizen", 100);
        addTimer("Web Worker Timer", 1800);
    }

    /**
     * Binds Back Key Events.
     * @private
     */
    function bindBackKeyEvents() {
        document.addEventListener("tizenhwkey", function(e) {
            if (e.keyName === "back") {
                var page = document.querySelectorAll(".active")[0],
                    pageId = page.id;

                try {
                    if (pageId === "page-main") {
                        tizen.application.getCurrentApplication().exit();
                    } else {
                        document.querySelector("#page-main").classList.toggle("active");
                        page.classList.toggle("active");
                    }
                } catch (ignore) {}
            }
        });
    }

    /**
     * Binds Button events.
     * @private
     */
    function bindButtonEvents() {
        document.querySelector("#main-add-btn").addEventListener("click", function() {
            clearAddPage();
            document.querySelector("#page-main").classList.toggle("active");
            document.querySelector("#page-add-timer").classList.toggle("active");
        });

        document.querySelector("#timer-add-btn").addEventListener("click", function() {
            var timerTitle = document.querySelector("#add-timer-title").value,
                timerHour = document.querySelector("#add-timer-hour").value,
                timerMin = document.querySelector("#add-timer-min").value,
                timerSec = document.querySelector("#add-timer-sec").value,
                timerTime = parseInt(timerHour * 3600) + parseInt(timerMin * 60) + parseInt(timerSec);

            if (timerTitle === "") {
                timerTitle = "No Title";
            }
            if (timerTime === 0) {
                alert("Please input time greater than 0");
            } else {
                addTimer(timerTitle, timerTime);
                document.querySelector("#page-main").classList.toggle("active");
                document.querySelector("#page-add-timer").classList.toggle("active");
            }
        });
    }

    /**
     * Binds onFocus, onBlur event to title input.
     * @private
     */
    function bindFocusEvents() {
        var addTimerPage = document.querySelector("#page-add-timer");

        document.querySelector("#add-timer-title").addEventListener("focus", function() {
            addTimerPage.querySelector(".page-header").style.display = "none";
            addTimerPage.querySelector(".page-footer").style.display = "none";
        });

        document.querySelector("#add-timer-title").addEventListener("blur", function() {
            addTimerPage.querySelector(".page-header").style.display = "block";
            addTimerPage.querySelector(".page-footer").style.display = "block";
        });
    }

    /**
     * Binds Default Events.
     * @private
     */
    function bindDefaultEvents() {
        bindBackKeyEvents();
        bindButtonEvents();
        bindFocusEvents();
    }

    /**
     * Initiates the application.
     * @private
     */
    function init() {
        bindDefaultEvents();
        setOptionList();
        setSampleTimers();
    }
    window.onload = init;
}());